def draw_all_pieces(piecelist,filename="results/pieces-raw.ipe",pointmarks=False):
    # stand-alone procedure for preparing illustrations
    # usually called as: draw_all_pieces(allpieces)
    # pointmarks is not implemented yet.
    out = startipefile(filename)
    for piece in piecelist:
        draw_piece(piece,out,pointmarks,labels=False)
    out.write('''<path stroke="black" pen="normal">
0 0 m
100 0 l
</path>
''') # draw a length scale
    out.write('''<path stroke="black" pen="normal">
0 0 m
500 0 l
</path>
''')
    finishipefile(out)

def draw_pieces_with_coordinates(piecelist):
    # stand-alone procedure
    # usually called as: draw_pieces_with_coordinates(allpieces[:2])
  for piece in piecelist:
    out=startipefile("results/DEFINITION-%s.ipe"%piece["filename"])
    draw_piece(piece,out,pointmarks=True,labels=True,
               scale=100.0/piece["scale"])
    finishipefile(out)

#### ===== AUXILIARY PROCEDURES FOR OTHER MODULES =====

def print_polygon(p,closed=True,options = 'stroke="black" pen="normal"'):
  """print a polygonal chain in ipe format"""
  if closed:
    final = ["h","</path>\n"]
  else:
    final = ["</path>\n"]
  return '\n'.join(
  ['<path %s>' % options,
  "%s %s m" % (p[0][0],p[0][1]) ] + ###!!
  map(lambda i: ("%s %s l" % p[i]), range(1,len(p))) + final
  )

def print_points(points,shape=2):
  """print a point set as marks in ipe format"""
  return ('<group layer="alpha">\n' +
   '\n'.join(
   map(lambda p:
     '<mark stroke="black" pos="%f %f" shape="%d" size="normal" pen="normal"/>'
     % (p[0],p[1],shape), points)) +
   '\n</group>\n'
  )

def print_labels(points,voffset=0.2):
  """print point coordinates for figures in ipe format"""
  return ('<group layer="alpha">\n' +
   '\n'.join(
   map(lambda p:
  '<text stroke="black" pos="%f %f" type="label" valign="bottom" halign="center"'
  ' transformable="no" size="small">$(%s,%s)$</text>\n' %
              (p[0], float(p[1])+voffset,
               p[0].normalize(), p[1].normalize()), points))+
   '\n</group>\n'
  )

def draw_piece(piece,out,pointmarks,labels,scale=1):
      out.write('<group layer="alpha" matrix="%5.3f 0 0 %5.3f">\n'
                '<group layer="alpha">\n'   % (scale,scale))
      p = [] # collect all points
      for part in piece["partlist"]:
        out.write(print_polygon(piece[part],closed=False))
        p.extend(piece[part])
      for (which, selections, base, baseindex) in piece["patterns"]:
        pbase = tuple(piece[base][baseindex])
        for (pos,letter) in selections:
            if pos>0: side="beginning"
            if pos<0: side="end"
        for (pos,letter) in selections:
            if pos>0: p2=piece[which][pos]
            elif pos<0: p2=piece[which][pos-1]
            elif side=="beginning": p2=piece[which][0]
            else: p2=piece[which][-1]
            # dash="dotted" 
            out.write('''<path stroke="black" pen="normal">
%s %s m
%s %s l
</path>
''' % (pbase + tuple(p2)))
      out.write('</group>\n')
      if pointmarks:
          out.write(print_points(p))
      if labels:
          out.write(print_labels(p))
      out.write('</group>\n')

def text_print_piece_coordinates(piece,out):
      out.write('## PIECE: %s\n' % piece["name"])
      for part in piece["partlist"]:
        out.write(' # %s, part "%s" of the boundary:\n'
                  % (piece["name"],part))
        for p in piece[part]:
           out.write('     (%5s, %5s)\n' % p)
      out.write('## END OF PIECE: %s.\n\n' % piece["name"])

def startipefile(outputfilename="test.ipe"):
    out = open(outputfilename,"w")
    out.write(
"""<ipe version="60023" creator="python-mwt routines" media="0 0 595 842">
<info created="D:20061207150701" modified="D:20061207150701" bbox="cropbox"/>
<page gridsize="16">
<layer name="alpha"/>
<view layers="alpha" active="alpha"/>
""")
    return out

def finishipefile(file):
    file.write("""</page>
</ipe>
""")
    file.close()

textformat = '<text stroke="black" pos="%f %f" type="label" valign="bottom"' +\
  ' transformable="no" size="large">%s</text>\n'

def print_triangles(poly,triangle_list,valuestring,pos=None):
  if pos==None:
    pos=poly[0]
  return ('''<group layer="alpha">
<path stroke="black" pen="normal">
%s %s m
%s %s l
</path>
''' % (poly[0][0],poly[0][1],poly[-1][0],poly[-1][1]))  + \
  '\n'.join( map(lambda triangle: print_one_triangle(poly,triangle),triangle_list)) + \
  '\n' + textformat % (pos[0],pos[1], valuestring) + \
  '</group>\n'

def print_one_triangle(p,t):
  return '''<path stroke="black" pen="normal">
%f %f m
%f %f l
%f %f l
</path>''' % tuple(p[t[0]]+p[t[1]]+p[t[2]])

### routines for solve_all_patterns: ####    
def show_solution(out,piece,number,p,sol,label):
     out.write('<group layer="alpha" '
               'matrix="1 0 0 1 %5.3f %5.3f">\n'
           % (piece.get("horishift",0)*(number-1),
             -piece["vertshift"]*(number-1)))
     out.write(print_polygon(p))
     out.write(print_triangles(p,sol,label,pos = piece["labelpos"]))
     out.write('</group>\n')

def write_table(tablefile,condensed_tablefile,
                name,resultlist,displaydigits):
  if tablefile:
    tablefile.write (r"""\noalign{\hrule}
    \multispan5\vrule\strut\raise3pt\hbox{\strut}\hfil %s\hfil\vrule\cr
    \noalign{\hrule}""" % name)

    for key,mult,cost,redcost,relredcost in resultlist:
      tablefile.write ( "%s &%d& %s & %s & %s \\cr\n" %
          (key,mult, cost.quantize_and_print(displaydigits),
                  redcost.quantize_and_print(displaydigits),
               relredcost.quantize_and_print(displaydigits)))
    tablefile.write (r"\noalign{\hrule}")
  if condensed_tablefile:
    if len(resultlist)!=4:    # if piece is the C-connection
      condensed_tablefile.write("%") # comment out this line
    condensed_tablefile.write("%20s & " % name +
      "& ".join(r[3].quantize_and_print(displaydigits)
                for r in resultlist)
      + "\\cr\n")
